#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################
#
# cisco_stackmember.py
# Member check for stacked Cisco Catalyst switches
# (c) Reinhard Waas 2022
#
######################################################

from typing import Any, Dict
from .agent_based_api.v1 import *
from .agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringByteTable,
)
from .utils.interfaces import render_mac_address

Section = Dict[str, Dict[str, Any]]

switchstates = {
    '1' : ('waiting', State.WARN),
    '2' : ('progressing',State.WARN),
    '3' : ('added',  State.OK),
    '4' : ('ready', State.OK),
    '5' : ('sdmMismatch', State.WARN),
    '6' : ('verMismatch', State.CRIT),
    '7' : ('featureMismatch', State.WARN),
    '8' : ('newMasterInit', State.WARN),
    '9' : ('provisioned', State.OK),
   '10' : ('invalid', State.CRIT),
   '11' : ('removed', State.CRIT),
}


switchroles = {
    '1' : 'Master',
    '2' : 'Member',
    '3' : 'Non-member',
    '4' : 'Stand-by',
}


def parse_cisco_stackmember(string_table: StringByteTable) -> Section:
    section: Section = {}
    for item in string_table:
        swnum = item[0]
        section[swnum] = {
            'swrole'   : switchroles[item[1]],
            'swstate'  : item[2],
            'swmac'    : render_mac_address(item[3])
            }
    return section




register.snmp_section(
    name = "snmp_cisco_stackmember",
    parse_function=parse_cisco_stackmember,
    fetch=SNMPTree(
        base=".1.3.6.1.4.1.9.9.500.1.2.1.1",       # CISCO-STACKWISE-MIB::cswSwitchInfoEntry
        oids=[
            "1",                                   # cswSwitchNumCurrent
            "3",                                   # cswSwitchRole
            "6",                                   # cswSwitchState
            OIDBytes("7"),                         # cswSwitchMacAddress
        ],
    ),
    detect = any_of(
        startswith(".1.3.6.1.2.1.1.2.0", ".1.3.6.1.4.1.9.1.1208"),        # C29xx
        startswith(".1.3.6.1.2.1.1.2.0", ".1.3.6.1.4.1.9.1.1068"),        # C2975
        startswith(".1.3.6.1.2.1.1.2.0", ".1.3.6.1.4.1.9.1.1093"),        # CBS3140
        startswith(".1.3.6.1.2.1.1.2.0", ".1.3.6.1.4.1.9.1.1094"),        # CBS3150
        startswith(".1.3.6.1.2.1.1.2.0", ".1.3.6.1.4.1.9.1.2277"),        # C35xx
        startswith(".1.3.6.1.2.1.1.2.0", ".1.3.6.1.4.1.9.1.2066"),        # C36xx
        startswith(".1.3.6.1.2.1.1.2.0", ".1.3.6.1.4.1.9.1.516"),         # C37xx
        startswith(".1.3.6.1.2.1.1.2.0", ".1.3.6.1.4.1.9.1.1745"),        # C38xx
        startswith(".1.3.6.1.2.1.1.2.0", ".1.3.6.1.4.1.9.1.2067"),        # C57xx
        startswith(".1.3.6.1.2.1.1.2.0", ".1.3.6.1.4.1.9.1.2494"),        # C9300
        startswith(".1.3.6.1.2.1.1.2.0", ".1.3.6.1.4.1.9.1.2593"),        # C9500
    ),
)



def discover_cisco_stackmember(section: Section) -> DiscoveryResult:
    if section.keys():
        yield Service(parameters=section)



def check_cisco_stackmember(params: Section, section: Section) -> CheckResult:
    members = section.keys()
    targetmembers = params.keys()
    common = sorted([ opt for opt in members if opt in targetmembers ])
    exceeding = sorted([ opt for opt in members if opt not in targetmembers ])
    missing = sorted([ opt for opt in targetmembers if opt not in members ])

    for sw in common:
        swstate = section[sw]['swstate']
        yield Result(state=switchstates[swstate][1], summary="%s switch %s (%s) %s" % ( \
                section[sw]['swrole'], sw, section[sw]['swmac'], switchstates[swstate][0] ))
    for sw in missing:
        yield Result(state=State.CRIT, summary="Missing %s switch %s (%s)" % ( \
                params[sw]['swrole'], sw, params[sw]['swmac'] ))
    for sw in exceeding:
        swstate = section[sw]['swstate']
        if switchstates[swstate][1] == State.CRIT:
            result = State.CRIT
        else:
            result = State.WARN
        yield Result(state=result, summary="Exceeding %s switch %s (%s) in state %s" % ( \
                section[sw]['swrole'], sw, section[sw]['swmac'], switchstates[swstate][0] ))




register.check_plugin(
    name="cisco_stackmember",
    sections=["snmp_cisco_stackmember"],
    service_name="Stack Members",
    discovery_function=discover_cisco_stackmember,
    check_function=check_cisco_stackmember,
    check_default_parameters={},         # needed to pass discovery parameters along!
)
